﻿using ProjectStormTrooper.CollectionGenericObjects;
using ProjectStormTrooper.Drawnings;
using ProjectStormTrooper.Exceptions;
using System.Text;

/// <summary>
/// Класс-хранилище коллекций
/// </summary>
/// <typeparam name="T"></typeparam>
public class StorageCollection<T>
    where T : DrawningWarPlane
{
    /// <summary>
    /// Словарь (хранилище) с коллекциями
    /// </summary>
    readonly Dictionary<string, ICollectionGenericObjects<T>> _storages;

    /// <summary>
    /// Возвращение списка названий коллекций
    /// </summary>
    public List<string> Keys => _storages.Keys.ToList();
    /// <summary>
	/// Ключевое слово, с которого должен начинаться файл
	/// </summary>
	private readonly string _collectionKey = "CollectionsStorage";

    /// <summary>
    /// Разделитель для записи ключа и значения элемента словаря
    /// </summary>
    private readonly string _separatorForKeyValue = "|";

    /// <summary>
    /// Разделитель для записей коллекции данных в файл
    /// </summary>
    private readonly string _separatorItems = ";";


    /// <summary>
    /// Конструктор
    /// </summary>
    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }

    /// <summary>
    /// Добавление коллекции в хранилище
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <param name="collectionType">тип коллекции</param>
    public void AddCollection(string name, CollectionType collectionType)
    {
        if(name == null || _storages.ContainsKey(name)) { return; }
        switch(collectionType)

        {
            case CollectionType.None:
                return;
            case CollectionType.Massive:
                _storages.Add(name, new MassiveGenericObjects<T> { });
                return;
            case CollectionType.List:
                _storages.Add(name, new ListGenericObjects<T> { });
                return;
        }

    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    public void DelCollection(string name)
    {
        if (name == null || !_storages.ContainsKey(name)) { return; }
        _storages.Remove(name);
    }

    /// <summary>
    /// Доступ к коллекции
    /// </summary>
    /// <param name="name">Название коллекции</param>
    /// <returns></returns>
    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
            if (name == null || !_storages.ContainsKey(name)) { return null; }
            return _storages[name];
        }
    }
    /// <summary>
    /// Сохранение информации по автомобилям в хранилище в файл
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
    public void SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            throw new ArgumentException("В хранилище отсутствуют коллекции для сохранения");

        }

        if (File.Exists(filename))
        {
            File.Delete(filename);
        }


        using FileStream fs = new(filename, FileMode.Create);
        using StreamWriter streamWriter = new StreamWriter(fs);
        streamWriter.Write(_collectionKey);
        foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
        {
            streamWriter.Write(Environment.NewLine);
            // не сохраняем пустые коллекции
            if (value.Value.Count == 0)
            {
                continue;
            }

            streamWriter.Write(value.Key);
            streamWriter.Write(_separatorForKeyValue);
            streamWriter.Write(value.Value.GetCollectionType);
            streamWriter.Write(_separatorForKeyValue);
            streamWriter.Write(value.Value.MaxCount);
            streamWriter.Write(_separatorForKeyValue);

            foreach (T? item in value.Value.GetItems())
            {
                string data = item?.GetDataForSave() ?? string.Empty;
                if (string.IsNullOrEmpty(data))
                {
                    continue;
                }

                streamWriter.Write(data);
                streamWriter.Write(_separatorItems);
            }
        }
    }

    /// <summary>
    /// Загрузка информации по автомобилям в хранилище из файла
    /// </summary>
    /// <param name="filename">Путь и имя файла</param>
    /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
    public void LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            throw new FileNotFoundException("Файл не существует");
        }

        using (FileStream fs = new(filename, FileMode.Open))
        {
            using StreamReader streamReader = new StreamReader(fs);
            string firstString = streamReader.ReadLine();
            if(firstString == null || firstString.Length == 0)
            {
                throw new ArgumentException("В файле нет данных");
            }
            if (!firstString.Equals(_collectionKey))
            {
                //если нет такой записи, то это не те данные
                throw new InvalidDataException("В файле неверные данные");
            }

            _storages.Clear();
            while (!streamReader.EndOfStream)
            {
                string[] record = streamReader.ReadLine().Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                if(record.Length != 4)
                {
                    continue;
                }
            ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType) ??
                throw new InvalidCastException("Не удалось определить тип коллекции:" + record[1]); ;


            collection.MaxCount = Convert.ToInt32(record[2]);

            string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
            foreach (string elem in set)
            {
                if (elem?.CreateDrawningWarPlane() is T warPlane)
                {
					try
					{
						if (!collection.Insert(warPlane))
						{
							throw new InvalidOperationException("Объект не удалось добавить в коллекцию: " + record[3]);
						}
					}
					catch (CollectionOverflowException ex)
					{
						throw new CollectionOverflowException("Коллекция переполнена", ex);
					}
                }
            }

            _storages.Add(record[0], collection);
            }
        }

    }

    /// <summary>
    /// Создание коллекции по типу
    /// </summary>
    /// <param name="collectionType"></param>
    /// <returns></returns>
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}